/* $Id: virtual_compat.c,v 1.1 2002/10/11 08:47:05 mikpe Exp $
 * Performance-monitoring counters driver.
 * Virtual perfctrs backwards compatibility support code.
 *
 * Copyright (C) 2002  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <asm/processor.h>

/* Implement ptrace_check_attach() for older kernels. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,15)
#include "compat.h"
#define task_has_cpu(tsk)	((tsk)->has_cpu)
int ptrace_check_attach(struct task_struct *child, int kill)
{
	if (!(child->ptrace & PT_PTRACED))
		return -ESRCH;

	if (child->p_pptr != current)
		return -ESRCH;

	if (!kill) {
		if (child->state != TASK_STOPPED)
			return -ESRCH;
#ifdef CONFIG_SMP
		/* Make sure the child gets off its CPU.. */
		for (;;) {
			task_lock(child);
			if (!task_has_cpu(child))
				break;
			task_unlock(child);
			do {
				if (child->state != TASK_STOPPED)
					return -ESRCH;
				barrier();
				cpu_relax();
			} while (task_has_cpu(child));
		}
		task_unlock(child);
#endif		
	}

	/* All systems go.. */
	return 0;
}
#endif	/* < 2.4.15 */

/* Simulate 2.5's struct file_system_type in 2.4. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#include <linux/module.h>
#define IN_VIRTUAL_COMPAT_C
#include "virtual_compat.h"

#define VPERFCTRFS_MAGIC	(('V'<<24)|('P'<<16)|('M'<<8)|('C'))

static int vperfctrfs_statfs(struct super_block *sb, struct statfs *buf)
{
	buf->f_type = VPERFCTRFS_MAGIC;
	buf->f_bsize = 1024;
	buf->f_namelen = 255;
	return 0;
}

static struct super_operations vperfctrfs_ops = {
	.statfs = vperfctrfs_statfs,
};

static struct super_block *vperfctrfs_read_super(struct super_block *sb, void *date, int silent)
{
	struct inode *root = new_inode(sb);
	if( !root )
		return NULL;
	root->i_mode = S_IFDIR | S_IRUSR | S_IWUSR;
	root->i_uid = root->i_gid = 0;
	root->i_atime = root->i_mtime = root->i_ctime = CURRENT_TIME;
	sb->s_blocksize = 1024;
	sb->s_blocksize_bits = 10;
	sb->s_magic = VPERFCTRFS_MAGIC;
	sb->s_op = &vperfctrfs_ops;
	sb->s_root = d_alloc(NULL, &(const struct qstr) { "vperfctrfs:", 11, 0 });
	if( !sb->s_root ) {
		iput(root);
		return NULL;
	}
	sb->s_root->d_sb = sb;
	sb->s_root->d_parent = sb->s_root;
	d_instantiate(sb->s_root, root);
	return sb;
}

static DECLARE_FSTYPE(actual_vperfctrfs_type, "vperfctrfs",
		      vperfctrfs_read_super, FS_NOMOUNT);

int perfctr_register_filesystem(struct perfctr_file_system_type *fake_fs_type)
{
	return register_filesystem(&actual_vperfctrfs_type);
}

void perfctr_unregister_filesystem(struct perfctr_file_system_type *fake_fs_type)
{
	unregister_filesystem(&actual_vperfctrfs_type);
}

struct vfsmount *perfctr_kern_mount(struct perfctr_file_system_type *fake_fs_type)
{
	return kern_mount(&actual_vperfctrfs_type);
}

#endif	/* < 2.5.0 */
